"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.WanXImageToVideo = void 0;
const GenericFunctions_1 = require("../shared/GenericFunctions");
const Constants_1 = require("../shared/Constants");
class WanXImageToVideo {
    constructor() {
        this.description = {
            displayName: 'PiAPI WanX Image to Video',
            name: 'wanXImageToVideo',
            icon: 'file:../piapi.svg',
            group: ['transform'],
            version: 1,
            description: 'Generate videos from images using PiAPI WanX',
            defaults: {
                name: 'WanX Image to Video',
            },
            inputs: ["main"],
            outputs: ["main"],
            credentials: [
                {
                    name: 'piAPIApi',
                    required: true,
                },
            ],
            properties: [
                {
                    displayName: 'Prompt',
                    name: 'prompt',
                    type: 'string',
                    typeOptions: {
                        rows: 4,
                    },
                    default: '',
                    required: true,
                    description: 'Descriptive prompt for video generation',
                },
                {
                    displayName: 'Negative Prompt',
                    name: 'negativePrompt',
                    type: 'string',
                    typeOptions: {
                        rows: 2,
                    },
                    default: '',
                    description: 'Things to exclude from the video generation',
                },
                {
                    displayName: 'Model',
                    name: 'model',
                    type: 'options',
                    options: Constants_1.WANX_MODELS.filter(model => model.value === 'img2video-14b'),
                    default: 'img2video-14b',
                    description: 'The WanX model to use for image-to-video generation',
                },
                {
                    displayName: 'Image Source',
                    name: 'imageSource',
                    type: 'options',
                    options: [
                        {
                            name: 'URL',
                            value: 'url',
                        },
                        {
                            name: 'Binary Data',
                            value: 'binaryData',
                        },
                    ],
                    default: 'url',
                    description: 'The source of the input image',
                },
                {
                    displayName: 'Image URL',
                    name: 'imageUrl',
                    type: 'string',
                    default: '',
                    required: true,
                    displayOptions: {
                        show: {
                            imageSource: ['url'],
                        },
                    },
                    description: 'Direct URL to the image (must be a publicly accessible image file)',
                },
                {
                    displayName: 'Binary Property',
                    name: 'binaryPropertyName',
                    type: 'string',
                    default: 'data',
                    required: true,
                    displayOptions: {
                        show: {
                            imageSource: ['binaryData'],
                        },
                    },
                    description: 'Name of the binary property containing the image data',
                },
                {
                    displayName: 'Aspect Ratio',
                    name: 'aspectRatio',
                    type: 'options',
                    options: [
                        {
                            name: 'Landscape (16:9)',
                            value: '16:9',
                        },
                        {
                            name: 'Portrait (9:16)',
                            value: '9:16',
                        }
                    ],
                    default: '16:9',
                    description: 'Aspect ratio of the generated video',
                },
                {
                    displayName: 'Wait for Completion',
                    name: 'waitForCompletion',
                    type: 'boolean',
                    default: false,
                    description: 'Wait for task to complete and return results',
                },
            ],
        };
    }
    async execute() {
        var _a;
        const items = this.getInputData();
        const returnData = [];
        for (let i = 0; i < items.length; i++) {
            const prompt = this.getNodeParameter('prompt', i);
            const negativePrompt = this.getNodeParameter('negativePrompt', i, '');
            const model = this.getNodeParameter('model', i, 'img2video-14b');
            const aspectRatio = this.getNodeParameter('aspectRatio', i);
            const waitForCompletion = this.getNodeParameter('waitForCompletion', i, false);
            const imageSource = this.getNodeParameter('imageSource', i);
            let imageBase64 = '';
            if (imageSource === 'url') {
                const imageUrl = this.getNodeParameter('imageUrl', i);
                try {
                    new URL(imageUrl);
                    try {
                        const imageResponse = await this.helpers.request({
                            method: 'GET',
                            url: imageUrl,
                            encoding: null,
                            resolveWithFullResponse: true,
                        });
                        const buffer = Buffer.from(imageResponse.body);
                        const contentType = imageResponse.headers['content-type'] || 'image/png';
                        imageBase64 = `data:${contentType};base64,${buffer.toString('base64')}`;
                    }
                    catch (error) {
                        throw new Error(`Failed to download image from URL: ${error.message}`);
                    }
                }
                catch (error) {
                    throw new Error(`Invalid image URL: ${error.message}`);
                }
            }
            else {
                const binaryPropertyName = this.getNodeParameter('binaryPropertyName', i);
                const binaryData = this.helpers.assertBinaryData(i, binaryPropertyName);
                if (binaryData.mimeType && !binaryData.mimeType.includes('image/')) {
                    throw new Error('The provided binary data is not an image. Please provide valid image data.');
                }
                if (binaryData.data) {
                    imageBase64 = `data:${binaryData.mimeType};base64,${binaryData.data}`;
                }
                else if (binaryData.url) {
                    try {
                        const imageResponse = await this.helpers.request({
                            method: 'GET',
                            url: binaryData.url,
                            encoding: null,
                            resolveWithFullResponse: true,
                        });
                        const buffer = Buffer.from(imageResponse.body);
                        const contentType = imageResponse.headers['content-type'] || 'image/png';
                        imageBase64 = `data:${contentType};base64,${buffer.toString('base64')}`;
                    }
                    catch (error) {
                        throw new Error(`Failed to download image from binary URL: ${error.message}`);
                    }
                }
                else {
                    throw new Error('Could not extract image from binary data. Missing URL or data.');
                }
            }
            const body = {
                model: 'Qubico/wanx',
                task_type: model,
                input: {
                    prompt,
                    negative_prompt: negativePrompt,
                    image: imageBase64,
                    aspect_ratio: aspectRatio,
                    video_resolution: '480P',
                },
            };
            try {
                const response = await GenericFunctions_1.piApiRequest.call(this, 'POST', '/api/v1/task', body);
                let taskResult = response;
                if (waitForCompletion && ((_a = response.data) === null || _a === void 0 ? void 0 : _a.task_id)) {
                    taskResult = await GenericFunctions_1.waitForTaskCompletion.call(this, response.data.task_id);
                }
                returnData.push({
                    json: taskResult,
                });
            }
            catch (error) {
                if (this.continueOnFail()) {
                    returnData.push({
                        json: {
                            error: error.message,
                        },
                    });
                    continue;
                }
                throw error;
            }
        }
        return [returnData];
    }
}
exports.WanXImageToVideo = WanXImageToVideo;
//# sourceMappingURL=WanXImageToVideo.node.js.map